using Automaise.Core;
using System.Collections.Generic;
using System.Globalization;

namespace CovidEuropeProjectConnector.Localizations
{
	public class CovidEuropeProjectLocResources : Dictionary<string, ICovidEuropeProjectLocalization>
	{
		public CovidEuropeProjectLocResources()
		{
			Add("pt", new CovidEuropeProjectPT_locale());
			Add("en", new CovidEuropeProjectEN_locale());
			Add("es", new CovidEuropeProjectES_locale());
			Add("fr", new CovidEuropeProjectFR_locale());
			Add("de", new CovidEuropeProjectDE_locale());
		}

		public static ICovidEuropeProjectLocalization Resources
		{
			get
			{
				var culture = CultureInfo.CurrentCulture;
				return GetLocResources(culture);
			}
		}
        
		private static ICovidEuropeProjectLocalization GetLocResources(CultureInfo culture)
		{
			if (Localizations.ContainsKey(culture.Name.ToLower()))
				return Localizations[culture.Name.ToLower()];
			else if (Localizations.ContainsKey(culture.TwoLetterISOLanguageName.ToLower()))
				return Localizations[culture.TwoLetterISOLanguageName.ToLower()];
			else if (Localizations.ContainsKey(culture.Name.ToLower().Split('-')[0]))
				return Localizations[culture.Name.ToLower().Split('-')[0]];
			else
				return Localizations[AnnaCoreConsts.ANNA_DEFAULT_LANG.ToLower()];
		}

		static private volatile CovidEuropeProjectLocResources localizations;

		private static object resbookRoot = new object();
		public static CovidEuropeProjectLocResources Localizations
		{
			get
			{
				if (localizations == null)
				{
					lock (resbookRoot)
					{
						if (localizations == null)
							localizations = new CovidEuropeProjectLocResources();
					}
				}
				return localizations;
			}
		}
	}
}

