using Automaise.Core.ClientDB;
using Automaise.Core.DbEntities;
using Automaise.Core.DTOs;
using Automaise.Core.Entities;
using Automaise.Core.ExternalConnector;
using Automaise.Core.Services;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using CovidEuropeProjectBase;
using CovidEuropeProjectConnector.Localizations;
using CovidEuropeProjectConnector.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace CovidEuropeProjectConnector
{
    /// <summary>
    /// GENERATED CODE. DO NOT EDIT.
    /// Example of a virtual employee. This class implements the employee interface methods
    /// </summary>
    public class CovidEuropeProjectConnector : ICovidEuropeProjectInterface
    {
        private class ConnectionData
        {
            public string APIUrl { get; set; }
        }

        private ConnectionData ConnData { set; get; }

        private readonly JsonSerializerSettings jsonSettings;

        protected AnnaSession Session { get; set; }

        private CovidEuropeProjectConnector() { }

        private Dictionary<string, string> GetNominatimHeaders()
        {
            Dictionary<string, string> connHeaders = new Dictionary<string, string>
            {
                ["user-agent"] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)",
                ["Referer"] = "http://www.microsoft.com"
            };

            return connHeaders;
        }

        public CovidEuropeProjectConnector(AnnaSession session, bool fullyInitizalized = true)
        {
            if (!fullyInitizalized)
            {
                return;
            }

            Session = session;
            jsonSettings = new JsonSerializerSettings
            {
                DateFormatString = "YYYY-MM-DD HH:mm:ss"
                //ContractResolver = new CovidEuropeProjectContractResolver()
            };

            try
            {
                ConnData = JsonConvert.DeserializeObject<ConnectionData>(AnnaCoreUtils.DecodeFromBase64(Session.MerchantObj.EmployeeSettings[CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID].Locations[0].ConnectionData));
            }
            catch { }
        }

        private Dictionary<string, string> GetHeaders()
        {
            return new Dictionary<string, string>();
        }

        public async Task<string> SearchCity(string city)
        {
            Logger.LOG(LogLevel.eDebug, null, "LeroyMerlinConnector: Invoking SearchCity...");
            string url = $"https://nominatim.openstreetmap.org/search?format=json&city={Uri.EscapeDataString(city)}";
            return await HttpUtils.GetAsync<string>(url, GetNominatimHeaders());
        }

        public Task<List<ListItem>> GetCountries()
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking GetCountries...");

            //List<ListItem> data = LocalCacheService.Instance.GetValue<List<ListItem>>(ViaCTTConsts.CACHE_CTT_COUNTRIES, Session.InboundUser.CustomerId);

            //if (data != null)
            //    return Task.FromResult(data);

            List<ListItem> data = new List<ListItem>
            {
                new ListItem { Id = "AF", Name = "Afeganist�o" },
                new ListItem { Id = "ZA", Name = "�frica do Sul" },
                new ListItem { Id = "AL", Name = "Alb�nia" },
                new ListItem { Id = "DE", Name = "Alemanha" },
                new ListItem { Id = "AD", Name = "Andorra" },
                new ListItem { Id = "AO", Name = "Angola" },
                new ListItem { Id = "AI", Name = "Anguilla" },
                new ListItem { Id = "AQ", Name = "Ant�rtica" },
                new ListItem { Id = "AG", Name = "Ant�gua/Barbuda" },
                new ListItem { Id = "AN", Name = "Antilhas Holand" },
                new ListItem { Id = "SA", Name = "Ar�bia Saudita" },
                new ListItem { Id = "DZ", Name = "Arg�lia" },
                new ListItem { Id = "AR", Name = "Argentina" },
                new ListItem { Id = "AM", Name = "Arm�nia" },
                new ListItem { Id = "AW", Name = "Aruba" },
                new ListItem { Id = "AU", Name = "Austr�lia" },
                new ListItem { Id = "AT", Name = "�ustria" },
                new ListItem { Id = "AZ", Name = "Azerbeij�o" },
                new ListItem { Id = "BS", Name = "Bahamas" },
                new ListItem { Id = "BH", Name = "Bahrein" },
                new ListItem { Id = "BD", Name = "Bangladesh" },
                new ListItem { Id = "BB", Name = "Barbados" },
                new ListItem { Id = "BE", Name = "B�lgica" },
                new ListItem { Id = "BZ", Name = "Belize" },
                new ListItem { Id = "BJ", Name = "Benin" },
                new ListItem { Id = "BM", Name = "Bermuda" },
                new ListItem { Id = "BY", Name = "Bielor�ssia" },
                new ListItem { Id = "BO", Name = "Bol�via" },
                new ListItem { Id = "BA", Name = "B�snia-Herzeg�." },
                new ListItem { Id = "BW", Name = "Botswana" },
                new ListItem { Id = "BR", Name = "Brasil" },
                new ListItem { Id = "BN", Name = "Brunei" },
                new ListItem { Id = "BG", Name = "Bulg�ria" },
                new ListItem { Id = "BF", Name = "Burkina" },
                new ListItem { Id = "BI", Name = "Burundi" },
                new ListItem { Id = "BT", Name = "But�o" },
                new ListItem { Id = "CV", Name = "Cabo Verde" },
                new ListItem { Id = "KH", Name = "Cambodja" },
                new ListItem { Id = "CA", Name = "Canad�" },
                new ListItem { Id = "KZ", Name = "Cazaquist�o" },
                new ListItem { Id = "TD", Name = "Chade" },
                new ListItem { Id = "CL", Name = "Chile" },
                new ListItem { Id = "CN", Name = "China" },
                new ListItem { Id = "CY", Name = "Chipre" },
                new ListItem { Id = "CO", Name = "Col�mbia" },
                new ListItem { Id = "KP", Name = "Coreia do Norte" },
                new ListItem { Id = "KR", Name = "Coreia do Sul" },
                new ListItem { Id = "CI", Name = "Costa do Marfim" },
                new ListItem { Id = "CR", Name = "Costa Rica" },
                new ListItem { Id = "HR", Name = "Cro�cia" },
                new ListItem { Id = "CU", Name = "Cuba" },
                new ListItem { Id = "DK", Name = "Dinamarca" },
                new ListItem { Id = "DJ", Name = "Djibouti" },
                new ListItem { Id = "DM", Name = "Dominica" },
                new ListItem { Id = "EG", Name = "Egipto" },
                new ListItem { Id = "SV", Name = "El Salvador" },
                new ListItem { Id = "AE", Name = "Emir.�rabes Uni" },
                new ListItem { Id = "EC", Name = "Equador" },
                new ListItem { Id = "ER", Name = "Eritreia" },
                new ListItem { Id = "SK", Name = "Eslov�quia" },
                new ListItem { Id = "SI", Name = "Eslov�nia" },
                new ListItem { Id = "ES", Name = "Espanha" },
                new ListItem { Id = "EE", Name = "Est�nia" },
                new ListItem { Id = "ET", Name = "Eti�pia" },
                new ListItem { Id = "US", Name = "EUA" },
                new ListItem { Id = "RU", Name = "Federa��o Russa" },
                new ListItem { Id = "FJ", Name = "Fidji" },
                new ListItem { Id = "PH", Name = "Filipinas" },
                new ListItem { Id = "FI", Name = "Finl�ndia" },
                new ListItem { Id = "FR", Name = "Fran�a" },
                new ListItem { Id = "GA", Name = "Gab�o" },
                new ListItem { Id = "GM", Name = "G�mbia" },
                new ListItem { Id = "GE", Name = "Ge�rgia" },
                new ListItem { Id = "GH", Name = "Ghana" },
                new ListItem { Id = "GI", Name = "Gibraltar" },
                new ListItem { Id = "GR", Name = "Gr�cia" },
                new ListItem { Id = "GD", Name = "Grenada" },
                new ListItem { Id = "GL", Name = "Gronel�ndia" },
                new ListItem { Id = "GP", Name = "Guadalupe" },
                new ListItem { Id = "GU", Name = "Guam" },
                new ListItem { Id = "GT", Name = "Guatemala" },
                new ListItem { Id = "GG", Name = "Guernsey" },
                new ListItem { Id = "GY", Name = "Guiana" },
                new ListItem { Id = "GF", Name = "Guiana Francesa" },
                new ListItem { Id = "GN", Name = "Guin�" },
                new ListItem { Id = "GQ", Name = "Guin� Equator." },
                new ListItem { Id = "GW", Name = "Guin�-Bissau" },
                new ListItem { Id = "HT", Name = "Haiti" },
                new ListItem { Id = "HM", Name = "Heard/Ilh.McDon" },
                new ListItem { Id = "NL", Name = "Holanda" },
                new ListItem { Id = "HN", Name = "Honduras" },
                new ListItem { Id = "HK", Name = "Hong Kong" },
                new ListItem { Id = "HU", Name = "Hungria" },
                new ListItem { Id = "MP", Name = "I.Mariana Norte" },
                new ListItem { Id = "TC", Name = "I.Turks, Name =  Caicos" },
                new ListItem { Id = "VI", Name = "I.Virgens Amer." },
                new ListItem { Id = "VG", Name = "I.Virgens Brit." },
                new ListItem { Id = "WF", Name = "I.Wallis e Fut." },
                new ListItem { Id = "UM", Name = "Ilh.Minor Outl." },
                new ListItem { Id = "IM", Name = "Ilha de Man" },
                new ListItem { Id = "AX", Name = "Ilhas Aland" },
                new ListItem { Id = "BV", Name = "Ilhas Bouvet" },
                new ListItem { Id = "KY", Name = "Ilhas Cayman" },
                new ListItem { Id = "CC", Name = "Ilhas Cocos" },
                new ListItem { Id = "KM", Name = "Ilhas Comoros" },
                new ListItem { Id = "CK", Name = "Ilhas Cook" },
                new ListItem { Id = "FO", Name = "Ilhas Faro�" },
                new ListItem { Id = "FK", Name = "Ilhas Malvinas" },
                new ListItem { Id = "MH", Name = "Ilhas Marshall" },
                new ListItem { Id = "CX", Name = "Ilhas Natal" },
                new ListItem { Id = "NF", Name = "Ilhas Norfolk" },
                new ListItem { Id = "PN", Name = "Ilhas Pitcairn" },
                new ListItem { Id = "SB", Name = "Ilhas Salom�o" },
                new ListItem { Id = "GS", Name = "Ilhas Sand. Sul" },
                new ListItem { Id = "TK", Name = "Ilhas Tokelau" },
                new ListItem { Id = "IN", Name = "�ndia" },
                new ListItem { Id = "IO", Name = "�ndias Ocident." },
                new ListItem { Id = "ID", Name = "Indon�sia" },
                new ListItem { Id = "IR", Name = "Ir�o" },
                new ListItem { Id = "IQ", Name = "Iraque" },
                new ListItem { Id = "IE", Name = "Irlanda" },
                new ListItem { Id = "IS", Name = "Isl�ndia" },
                new ListItem { Id = "IL", Name = "Israel" },
                new ListItem { Id = "IT", Name = "It�lia" },
                new ListItem { Id = "JM", Name = "Jamaica" },
                new ListItem { Id = "JP", Name = "Jap�o" },
                new ListItem { Id = "JE", Name = "Jersey" },
                new ListItem { Id = "JO", Name = "Jord�nia" },
                new ListItem { Id = "KG", Name = "Kirghizist�o" },
                new ListItem { Id = "KI", Name = "Kiribati" },
                new ListItem { Id = "XZ", Name = "Kosovo" },
                new ListItem { Id = "KW", Name = "Kuwait" },
                new ListItem { Id = "LA", Name = "Laos" },
                new ListItem { Id = "LS", Name = "Lesotho" },
                new ListItem { Id = "LV", Name = "Let�nia" },
                new ListItem { Id = "LB", Name = "L�bano" },
                new ListItem { Id = "LR", Name = "Lib�ria" },
                new ListItem { Id = "LY", Name = "L�bia" },
                new ListItem { Id = "LI", Name = "Liechtenstein" },
                new ListItem { Id = "LT", Name = "Litu�nia" },
                new ListItem { Id = "LU", Name = "Luxemburgo" },
                new ListItem { Id = "MO", Name = "Macau" },
                new ListItem { Id = "MK", Name = "Maced�nia" },
                new ListItem { Id = "MG", Name = "Madag�scar" },
                new ListItem { Id = "MY", Name = "Mal�sia" },
                new ListItem { Id = "MW", Name = "Malawi" },
                new ListItem { Id = "MV", Name = "Maldivas" },
                new ListItem { Id = "ML", Name = "Mali" },
                new ListItem { Id = "MT", Name = "Malta" },
                new ListItem { Id = "MA", Name = "Marrocos" },
                new ListItem { Id = "MQ", Name = "Martinica" },
                new ListItem { Id = "MU", Name = "Maur�cias" },
                new ListItem { Id = "MR", Name = "Maurit�nia" },
                new ListItem { Id = "YT", Name = "Mayotte" },
                new ListItem { Id = "MX", Name = "M�xico" },
                new ListItem { Id = "FM", Name = "Micron�sia" },
                new ListItem { Id = "MZ", Name = "Mo�ambique" },
                new ListItem { Id = "MD", Name = "Mold�via" },
                new ListItem { Id = "MC", Name = "M�naco" },
                new ListItem { Id = "MN", Name = "Mong�lia" },
                new ListItem { Id = "ME", Name = "Montenegro" },
                new ListItem { Id = "MS", Name = "Montserrat" },
                new ListItem { Id = "NA", Name = "Nam�bia" },
                new ListItem { Id = "NR", Name = "Nauru" },
                new ListItem { Id = "NP", Name = "Nepal" },
                new ListItem { Id = "NI", Name = "Nicar�gua" },
                new ListItem { Id = "NE", Name = "N�ger" },
                new ListItem { Id = "NG", Name = "Nig�ria" },
                new ListItem { Id = "NU", Name = "Niue" },
                new ListItem { Id = "NO", Name = "Noruega" },
                new ListItem { Id = "NC", Name = "Nova Caled�nia" },
                new ListItem { Id = "NZ", Name = "Nova Zel�ndia" },
                new ListItem { Id = "PW", Name = "Palau" },
                new ListItem { Id = "PA", Name = "Panam�" },
                new ListItem { Id = "PG", Name = "Papua Nov.Guin�" },
                new ListItem { Id = "PK", Name = "Paquist�o" },
                new ListItem { Id = "PY", Name = "Paraguai" },
                new ListItem { Id = "PE", Name = "Peru" },
                new ListItem { Id = "PF", Name = "Polin�sia Franc" },
                new ListItem { Id = "PL", Name = "Pol�nia" },
                new ListItem { Id = "PR", Name = "Porto Rico" },
                new ListItem { Id = "PT", Name = "Portugal" },
                new ListItem { Id = "QA", Name = "Quatar" },
                new ListItem { Id = "KE", Name = "Qu�nia" },
                new ListItem { Id = "GB", Name = "Reino Unido" },
                new ListItem { Id = "CF", Name = "Rep.Centro-Afr." },
                new ListItem { Id = "DO", Name = "Rep.Dominicana" },
                new ListItem { Id = "CM", Name = "Rep�b.Camar�es" },
                new ListItem { Id = "CD", Name = "Rep�b.Dem.Congo" },
                new ListItem { Id = "CG", Name = "Rep�b.Pop.Congo" },
                new ListItem { Id = "CZ", Name = "Rep�blica Checa" },
                new ListItem { Id = "RE", Name = "Reuni�o" },
                new ListItem { Id = "RO", Name = "Rom�nia" },
                new ListItem { Id = "RW", Name = "Ruanda" },
                new ListItem { Id = "KN", Name = "S.Crist.e N�vis" },
                new ListItem { Id = "ST", Name = "S.Tom� e Pr�nc." },
                new ListItem { Id = "EH", Name = "Saara Ocidental" },
                new ListItem { Id = "BL", Name = "Saint Barth�lemy" },
                new ListItem { Id = "MF", Name = "Saint Martin" },
                new ListItem { Id = "AS", Name = "Samoa Americana" },
                new ListItem { Id = "WS", Name = "Samoa Ocidental" },
                new ListItem { Id = "SM", Name = "San Marino" },
                new ListItem { Id = "SH", Name = "Santa Helena" },
                new ListItem { Id = "LC", Name = "Santa L�cia" },
                new ListItem { Id = "VC", Name = "S�o Vicente" },
                new ListItem { Id = "SN", Name = "Senegal" },
                new ListItem { Id = "SL", Name = "Serra Leoa" },
                new ListItem { Id = "RS", Name = "S�rvia" },
                new ListItem { Id = "CS", Name = "Servia Monteneg" },
                new ListItem { Id = "SC", Name = "Seychelles" },
                new ListItem { Id = "SG", Name = "Singapura" },
                new ListItem { Id = "SY", Name = "S�ria" },
                new ListItem { Id = "SO", Name = "Som�lia" },
                new ListItem { Id = "LK", Name = "Sri Lanka" },
                new ListItem { Id = "PM", Name = "StPier., Name = Miquel." },
                new ListItem { Id = "SZ", Name = "Suazil�ndia" },
                new ListItem { Id = "SD", Name = "Sud�o" },
                new ListItem { Id = "SE", Name = "Su�cia" },
                new ListItem { Id = "CH", Name = "Su��a" },
                new ListItem { Id = "OM", Name = "Sultanato Oman" },
                new ListItem { Id = "SR", Name = "Suriname" },
                new ListItem { Id = "SJ", Name = "Svalbard" },
                new ListItem { Id = "TJ", Name = "Tadjiquist�o" },
                new ListItem { Id = "TH", Name = "Tail�ndia" },
                new ListItem { Id = "TW", Name = "Taiwan" },
                new ListItem { Id = "TZ", Name = "Tanz�nia" },
                new ListItem { Id = "TF", Name = "Ter.Fr.Antartic" },
                new ListItem { Id = "PS", Name = "Territ�rios Palestinianos Ocupados" },
                new ListItem { Id = "TL", Name = "Timor Leste" },
                new ListItem { Id = "TG", Name = "Togo" },
                new ListItem { Id = "TO", Name = "Tonga" },
                new ListItem { Id = "TT", Name = "Trinidad, Name = Tobago" },
                new ListItem { Id = "TN", Name = "Tun�sia" },
                new ListItem { Id = "TM", Name = "Turquemenist�o" },
                new ListItem { Id = "TR", Name = "Turquia" },
                new ListItem { Id = "TV", Name = "Tuvalu" },
                new ListItem { Id = "UA", Name = "Ucr�nia" },
                new ListItem { Id = "UG", Name = "Uganda" },
                new ListItem { Id = "MM", Name = "Uni�o Myanmar" },
                new ListItem { Id = "UY", Name = "Uruguai" },
                new ListItem { Id = "UZ", Name = "Uzbequist�o" },
                new ListItem { Id = "VU", Name = "Vanuatu" },
                new ListItem { Id = "VA", Name = "Vaticano" },
                new ListItem { Id = "VE", Name = "Venezuela" },
                new ListItem { Id = "VN", Name = "Vietname" },
                new ListItem { Id = "YE", Name = "Y�men" },
                new ListItem { Id = "ZM", Name = "Z�mbia" },
                new ListItem { Id = "ZW", Name = "Zimbabwe" }
            };

            //LocalCacheService.Instance.AddValue(ViaCTTConsts.CACHE_CTT_COUNTRIES, Session.InboundUser.CustomerId, data, 1440);

            return Task.FromResult(data);
        }

        public Task<List<ListItem>> GetDistricts()
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking GetDistricts...");

            //List<ListItem> data = LocalCacheService.Instance.GetValue<List<ListItem>>(ViaCTTConsts.CACHE_CTT_COUNTRIES, Session.InboundUser.CustomerId);

            //if (data != null)
            //    return Task.FromResult(data);

            List<ListItem> data = new List<ListItem>
            {
                new ListItem { Id = "ZW", Name = "Aveiro" },
                new ListItem { Id = "ZW", Name = "Beja" },
                new ListItem { Id = "ZW", Name = "Braga" },
                new ListItem { Id = "ZW", Name = "Bragan�a" },
                new ListItem { Id = "ZW", Name = "Castelo Branco" },
                new ListItem { Id = "ZW", Name = "Coimbra" },
                new ListItem { Id = "ZW", Name = "�vora" },
                new ListItem { Id = "ZW", Name = "Faro" },
                new ListItem { Id = "ZW", Name = "Guarda" },
                new ListItem { Id = "ZW", Name = "Leiria" },
                new ListItem { Id = "ZW", Name = "Lisboa" },
                new ListItem { Id = "ZW", Name = "Portalegre" },
                new ListItem { Id = "ZW", Name = "Porto" },
                new ListItem { Id = "ZW", Name = "Santar�m" },
                new ListItem { Id = "ZW", Name = "Set�bal" },
                new ListItem { Id = "ZW", Name = "Viana do Castelo" },
                new ListItem { Id = "ZW", Name = "Vila Real" },
                new ListItem { Id = "ZW", Name = "Viseu" },
                new ListItem { Id = "ZW", Name = "Ilha da Madeira" },
                new ListItem { Id = "ZW", Name = "Ilha de Porto Santo" },
                new ListItem { Id = "ZW", Name = "Ilha de Santa Maria" },
                new ListItem { Id = "ZW", Name = "Ilha de S�o Miguel" },
                new ListItem { Id = "ZW", Name = "Ilha de S�o Miguel" },
                new ListItem { Id = "ZW", Name = "Ilha Terceira" },
                new ListItem { Id = "ZW", Name = "Ilha Graciosa" },
                new ListItem { Id = "ZW", Name = "Ilha de S�o Jorge" },
                new ListItem { Id = "ZW", Name = "Ilha do Pico" },
                new ListItem { Id = "ZW", Name = "Ilha do Faial" },
                new ListItem { Id = "ZW", Name = "Ilha das Flores" },
                new ListItem { Id = "ZW", Name = "Ilha do Corvo" },
            };

            return Task.FromResult(data);
        }

        public Task<List<ListItem>> MRCQuestionaire()
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking MRCQuestionaire...");

            List<ListItem> data = new List<ListItem>
            {
                new ListItem { Id = "0", Name = CovidEuropeProjectLocResources.Resources.SCALE1 },
                new ListItem { Id = "1", Name = CovidEuropeProjectLocResources.Resources.SCALE2 },
                new ListItem { Id = "2", Name = CovidEuropeProjectLocResources.Resources.SCALE3 },
                new ListItem { Id = "3", Name = CovidEuropeProjectLocResources.Resources.SCALE4 },
                new ListItem { Id = "4", Name = CovidEuropeProjectLocResources.Resources.SCALE5 },
            };

            return Task.FromResult(data);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public ConnectorInfo ConnectorInfo()
        {
            return new ConnectorInfo
            {
                Id = CovidEuropeProjectConsts.CovidEuropeProject_EMPLOYEE_ID,
                Name = "CovidEuropeProject Connector",
                FullFeatured = true,
                RegistrationRequired = true,
                IdEntityType = EntityType.Number,
                IdValidatiorRegEx = "^[0-9]{8,14}$",
            };
        }

        /// <summary>
        /// Method invoked to verify if the connection is active
        /// </summary>
        /// <returns>Returns a boolean depending on the connection state</returns>
        public Task<bool> CheckConnection()
        {
            throw new NotImplementedException();
        }

        Task<List<Field>> IExternalConnector.GetSetupFields(LocationSettings location, string extraParam)
        {
            throw new NotImplementedException();
        }

        public Task<List<WorkerSettingsDTO>> GetCustomWorkers(string merchantId)
        {
            return Task.FromResult(new List<WorkerSettingsDTO>());
        }

        Task<(AuthData, AnnaSession.MerchantCustomer)> IExternalConnector.AuthenticateCustomer(FormDataResult authenticateIn)
        {
            throw new NotImplementedException();
        }

        public async Task<int> GetCovidResults(string infectedConfirmation, string fever, string cough, string breathing)
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking GetCovidResults...");

            return await HttpUtils.GetAsync<int>(ConnData.APIUrl + $"?geo_contConfirmed={infectedConfirmation}&sympt_fever={fever}&sympt_cough={cough}&sympt_breath={breathing}", GetHeaders(), jsonSettings);
        }

        public async Task<int> SendCovidResults(CovidData covidData)
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking SendCovidResults...");

            return await HttpUtils.PostAsync<int>(ConnData.APIUrl, covidData, GetHeaders(), jsonSettings);
        }

        public async Task<CovidResult> SendCovidResultsV2(CovidDataV2 covidData)
        {
            Logger.LOG(LogLevel.eDebug, null, "CovidEuropeProjectConnector: Invoking SendCovidResults...");

            return MapMultiLangResults(await HttpUtils.PostAsync<CovidResult>(ConnData.APIUrl, covidData, GetHeaders(), jsonSettings));
        }

        private CovidResult MapMultiLangResults(CovidResult result)
        {
            switch (result.level)
            {
                case 1:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION1;
                    break;
                case 2:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION2;
                    break;
                case 3:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION3;
                    break;
                case 4:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION4;
                    break;
                case 5:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION5;
                    break;
                case 6:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION6;
                    break;
                case 7:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION7;
                    break;
                case 8:
                    result.message = CovidEuropeProjectLocResources.Resources.RECOMMENDATION8;
                    break;
            }

            return result;
        }
    }
}

