using Automaise.Core.DbEntities;
using Automaise.Core.DTOs;
using Automaise.Core.Entities;
using Automaise.Core.Entities.Flows;
using Automaise.Core.Localizations;
using Automaise.Core.ClientDB;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows;
using Automaise.Flows.AgentFacade;
using Automaise.Flows.Utils;
using CovidEuropeProjectEdge.CovidEuropeProject.Workers;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using static Automaise.Core.Session.AnnaSession;
using Automaise.Flows.Base.IntentWorkers;
using CovidEuropeProjectConnector.Localizations;
using CovidEuropeProjectBase;
using static CovidEuropeProjectBase.CovidEuropeProjectConsts;
using Automaise.Core.Services;
using MongoDB.Driver;
using System.Linq;
using Automaise.Flows.FAQs.Base;
using Automaise.Flows.FAQs.Extended;
using Automaise.Flows.FAQs.Workers;

namespace CovidEuropeProjectEdge.CovidEuropeProject
{
    /// <summary>
    /// Entry point for the virtual employee interaction
    /// </summary>
    public class CovidEuropeProjectFacade : EmployeeFacadeBase
    {
        /// <summary>
        /// Initialize the specific employee worker
        /// </summary>
        public CovidEuropeProjectFacade()
        {
            CommonLocResources.Localizations["pt"] = new CommonCovidEuropeProjectOverride_PT();
            CommonLocResources.Localizations["es"] = new CommonCovidEuropeProjectOverride_ES();
            CommonLocResources.Localizations["fr"] = new CommonCovidEuropeProjectOverride_FR();
            CommonLocResources.Localizations["en"] = new CommonCovidEuropeProjectOverride_EN();
            CommonLocResources.Localizations["de"] = new CommonCovidEuropeProjectOverride_DE();

            ExternalConnectors.Add(
                CovidEuropeProjectConsts.COVID_RECOMMENDATIONS_EXPERT, new Dictionary<int, Type>
                {
                    {FAQsConsts.BASE_CONNECTOR_ID, typeof(AutomaiseFAQsConnector) }
                }
            );

            ExternalConnectors.Add(
                CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID, new Dictionary<int, Type>
                {
                    {Connectors.CONNECTOR_ID, typeof(CovidEuropeProjectConnector.CovidEuropeProjectConnector) }
                }
            );

            IntentWorkers.Remove(FAQsConsts.VIRTUAL_EMPLOYEE_ID);

            IntentWorkers.Add(
                FAQsConsts.VIRTUAL_EMPLOYEE_ID, new Dictionary<int, TypeAndArgs>
                {
                    {AnnaFlowsConsts.FALLBACK_INTENTWORKER, new TypeAndArgs { ClassType = typeof(FAQsWorker),
                        ClassArgs = new object[] { new FAQsWorker.FAQsWorkerSettings { AddOkButton = false, AskWhatElse = true } } } }
                });

            IntentWorkers.Add(
                VIRTUAL_EMPLOYEE_ID, new Dictionary<int, TypeAndArgs>
                {
                    {SCREENING, new TypeAndArgs { ClassType = typeof(ScreeningWorker) } },
                    {AnnaFlowsConsts.FALLBACK_INTENTWORKER, new TypeAndArgs { ClassType = typeof(AutomationFlowWorker) } }
                });

            IntentWorkers.Add(
                COVID_RECOMMENDATIONS_EXPERT, new Dictionary<int, TypeAndArgs>
                {
                    {AnnaFlowsConsts.FALLBACK_INTENTWORKER, new TypeAndArgs { ClassType = typeof(FAQsWorker), ClassArgs = new object[] {
                        new FAQsWorker.FAQsWorkerSettings { AddOkButton = false, AskWhatElse = true, EmployeeToken = COVID_RECOMMENDATIONS_EXPERT } } } }
                });
        }

        /// <summary>
        /// Method invoked to display the main menu
        /// </summary>
        /// <param name="session">Session object for the user/customer interating with the virtual employee</param>
        /// <param name="message">Displayed message to the user/customer</param>
        /// <param name="withSalutation">Display salutation if desired. Default value is "false"</param>
        /// <returns>Returns a list of StepResult</returns>
        public override async Task<List<StepResult>> EmployeeMainMenu(AnnaSession session, string message, bool withSalutation = false)
        {
            List<StepResult> results = new List<StepResult>();

            StepResult result = new StepResult
            {
                WaitMsBefore = 50
            };

            if (!string.IsNullOrWhiteSpace(message))
            {
                result.Message = message;
            }
            else
            {
                result.Message = AnnaCoreUtils.PickOneFromMany(CommonLocResources.Resources.WHATCANIDO1, CommonLocResources.Resources.WHATCANIDO2);
            }

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButton(
                    new IntentControlData { MainIntent = MainIntents.EmployeeIntent, EmployeeIntent = SCREENING, VEmployee = VIRTUAL_EMPLOYEE_ID },
                    CovidEuropeProjectLocResources.Resources.SCREENING,
                    BKT_SCREENING,
                    new List<string>(),
                    MainIntents.EmployeeIntent,
                    new List<int> { SCREENING },
                    null, StepResultUtils.VerticalButtonType.eMainMenu));

            FormState stateFAQS = new FormState(GetType())
            {
                IntentControl = session.IntentControl,
                MerchantId = session.MerchantId,
                ChatId = session.SessionId,
                MsgType = MessageType.eUndefined,
                Parameters = new Dictionary<string, string>
                {
                    { "optoken", "FAQS" },
                }
            };

            string faqsFormUrl = await WebPartsHelper.BuildWebPartURL(session, WebPartsHelper.WebParts.eFAQs, stateFAQS, true);

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButtonWithURL(
                    CommonLocResources.Resources.QUESTIONSANSWERS,
                    faqsFormUrl,
                    true,
                    StepResultUtils.WebViewSize.eTall,
                    StepResultUtils.VerticalButtonType.eMainMenu));

            FormState stateRecomm = new FormState(GetType())
            {
                IntentControl = session.IntentControl,
                MerchantId = session.MerchantId,
                ChatId = session.SessionId,
                MsgType = MessageType.eUndefined,
                Parameters = new Dictionary<string, string>
                {
                    { "optoken", RECOMMENDATIONS },
                }
            };

            string recommFormUrl = await WebPartsHelper.BuildWebPartURL(session, WebPartsHelper.WebParts.eFAQs, stateRecomm, true);

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButtonWithURL(
                    CovidEuropeProjectLocResources.Resources.RECOMENDATIONS,
                    recommFormUrl,
                    true,
                    StepResultUtils.WebViewSize.eTall,
                    StepResultUtils.VerticalButtonType.eMainMenu));

            results.Add(result);

            return results;
        }

        public override async Task<FAQFormDTO> GetEmployeeFAQsList(AnnaSession session, FormState formState)
        {
            FAQFormDTO faqForm = null;

            if (formState.Parameters["optoken"].Equals("FAQS"))
            {
                IFAQsConnector faqsConnector = ExternalConnectorsFactory<IFAQsConnector>(session, FAQsConsts.VIRTUAL_EMPLOYEE_ID);

                faqForm = new FAQFormDTO()
                {
                    FaqItems = await faqsConnector.GetFAQsList(session.MerchantObj.EmployeeSettings[FAQsConsts.VIRTUAL_EMPLOYEE_ID].AIModel),
                    Title = CommonLocResources.Resources.FREQUENTLYASKEDQUESTIONS
                };
                MetricsService.METRIC(session.MerchantId, session.SessionId, CommonLocResources.Resources.FREQUENTLYASKEDQUESTIONS, MetricType.Default, session);
            }
            else if (formState.Parameters["optoken"].Equals(RECOMMENDATIONS))
            {
                IFAQsConnector faqsConnector = ExternalConnectorsFactory<IFAQsConnector>(session, COVID_RECOMMENDATIONS_EXPERT);

                faqForm = new FAQFormDTO()
                {
                    FaqItems = await faqsConnector.GetFAQsList(session.MerchantObj.EmployeeSettings[COVID_RECOMMENDATIONS_EXPERT].AIModel),
                    Title = CovidEuropeProjectLocResources.Resources.COVID19RECOMMENDATIONS
                };
                MetricsService.METRIC(session.MerchantId, session.SessionId, CovidEuropeProjectLocResources.Resources.COVID19RECOMMENDATIONS, MetricType.Default, session);
            }

            return faqForm;
        }

        /// <summary>
        /// Method invoked to display the metrics of the users/customers interaction between the virtual employee
        /// </summary>
        /// <param name="chartviews">Used to change the chart view periodicity</param>
        /// <param name="mtotals"></param>
        /// <param name="merchant">Object for the merchant data</param>
        /// <param name="merchDate"></param>
        /// <returns>Returns the data to display the charts</returns>
        public override List<VemployeeChartDataV3DTO> GetEmployeeCharts(AutoDictionary<string, ChartPeriod> chartviews, MetricTotals mtotals, Merchant merchant, DateTime merchDate)
        {
            List<VemployeeChartDataV3DTO> covid19Charts = new List<VemployeeChartDataV3DTO>();

            var clientsDB = ClientDbService.DatabaseInstance(merchant.MerchantId, AUTOMAISE_PARTNERS_KEY);

            //Pre Check results observed
            List<string> preCheckResultsMetrics = new List<string>
            {
                RECOMMEND_PREVENTION,
                RECOMMEND_MONITORING,
                RECOMMEND_ACTION
            };

            ChartDataV3 preCheckResultsMetricsChartData = new ChartDataV3()
            {
                HasDropdown = true,
                HasLegend = true,
                BootstrapClass = preCheckResultsMetrics.Count >= 7 ? "col-md-12" : "col-md-6",
                Height = "120",
                Id = "preCheckMetrics",
                IsToUseFreeUnit = false,
                Title = CovidEuropeProjectLocResources.Resources.SCREENINGRESULT,
                Type = "line",
                ValueToFixed = 0
            };

            covid19Charts.Add(new VemployeeChartDataV3DTO()
            {
                ChartData = preCheckResultsMetricsChartData,
                Metrics = preCheckResultsMetrics
            });

            //FAQs Metrics
            List<string> triggeredFaqsMetrics = new List<string>();

            Guid aiModelId = merchant.EmployeeSettings[FAQsConsts.VIRTUAL_EMPLOYEE_ID].AIModel;

            LocalCacheService.Instance.GetFAQSettings(merchant, aiModelId).CategoriesList.ForEach(element =>
            {
                triggeredFaqsMetrics.Add(element.CategoryTitle);
            });

            ChartDataV3 triggeredFaqsChartData = new ChartDataV3()
            {
                HasDropdown = true,
                HasLegend = true,
                BootstrapClass = triggeredFaqsMetrics.Count >= 7 ? "col-md-12" : "col-md-6",
                Height = triggeredFaqsMetrics.Count > 10 ? "180" : "120",
                Id = "triggeredFaqs",
                IsToUseFreeUnit = false,
                Title = CommonLocResources.Resources.QUESTIONSANSWERS,
                Type = "line",
                ValueToFixed = 0
            };

            covid19Charts.Add(new VemployeeChartDataV3DTO()
            {
                ChartData = triggeredFaqsChartData,
                Metrics = triggeredFaqsMetrics
            });

            //Recommendation Metrics
            List<string> triggeredRecommendationMetrics = new List<string>();

            Guid aiRecommendationsModelId = merchant.EmployeeSettings[CovidEuropeProjectConsts.COVID_RECOMMENDATIONS_EXPERT].AIModel;

            LocalCacheService.Instance.GetFAQSettings(merchant, aiRecommendationsModelId).CategoriesList.ForEach(element =>
            {
                triggeredRecommendationMetrics.Add(element.CategoryTitle);
            });

            ChartDataV3 triggeredRecommendationsChartData = new ChartDataV3()
            {
                HasDropdown = true,
                HasLegend = true,
                BootstrapClass = triggeredRecommendationMetrics.Count >= 7 ? "col-md-12" : "col-md-6",
                Height = triggeredRecommendationMetrics.Count > 10 ? "180" : "120",
                Id = "recommendationsFaqs",
                IsToUseFreeUnit = false,
                Title = CovidEuropeProjectLocResources.Resources.RECOMENDATIONS,
                Type = "line",
                ValueToFixed = 0
            };

            covid19Charts.Add(new VemployeeChartDataV3DTO()
            {
                ChartData = triggeredRecommendationsChartData,
                Metrics = triggeredRecommendationMetrics
            });

            //Webchat URLs counter
            var urltokens = clientsDB.Metrics(merchant.MerchantId)
                .AsQueryable().Where(p => p.MetricEvent.Equals(MetricEvent.MsgReceived.ToString()) && p.Param1 != null && p.EventDate > DateTime.Now.AddDays(-30))
                .Select(p => p.Param1).ToList();

            var urltokenMetrics = urltokens.GroupBy(p => p)
                .Select(p => p.First())
                .ToList();

            ChartDataV3 triggeredURLtokenChartData = new ChartDataV3()
            {
                HasDropdown = true,
                HasLegend = true,
                BootstrapClass = "col-md-12",
                Height = "250",
                Id = "webchatSources",
                IsToUseFreeUnit = false,
                Title = "WebChat Sources",
                Type = "line",
                ValueToFixed = 0,
                ExcludeEmptyLabels = true
            };

            covid19Charts.Add(new VemployeeChartDataV3DTO
            {
                ChartData = triggeredURLtokenChartData,
                Metrics = urltokenMetrics
            });

            //FAQs and Recommendations browse
            List<string> faqsBrowseMetrics = new List<string>
            {
                CommonLocResources.Resources.FREQUENTLYASKEDQUESTIONS,
                CovidEuropeProjectLocResources.Resources.COVID19RECOMMENDATIONS
            };

            ChartDataV3 faqsBrowseMetricsChartData = new ChartDataV3()
            {
                HasDropdown = true,
                HasLegend = true,
                BootstrapClass = "col-md-6",
                Height = "120",
                Id = "faqsBrowseMetrics",
                IsToUseFreeUnit = false,
                Title = CommonLocResources.Resources.VIEWS,
                Type = "line",
                ValueToFixed = 0
            };

            covid19Charts.Add(new VemployeeChartDataV3DTO
            {
                ChartData = faqsBrowseMetricsChartData,
                Metrics = faqsBrowseMetrics
            });

            return covid19Charts;
        }

        /// <summary>
        /// Method invoked to send the necessary data to feed dynamically generic webviews
        /// </summary>
        /// <param name="session">Session object for the user/customer interating with the virtual employee</param>
        /// <param name="optoken">Parameter to switch between operations</param>
        /// <param name="opdata">Object with the necessary data to be sent</param>
        /// <returns></returns>
        public override Task<object> GetEmployeeData(AnnaSession session, string optoken, string opdata)
        {
            return Task.FromResult<object>(new List<OptionToSelect>());
        }

        public override Task<object> PostEmployeeData(AnnaSession session, string optoken, object opdata)
        {
            return Task.FromResult<object>(new List<OptionToSelect>());
        }
    }
}

