using Automaise.Core.Entities.Flows;
using Automaise.Core.Localizations;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows.IntentWorkers.ActionSteps;
using Automaise.Flows.Utils;
using CovidEuropeProjectBase;
using CovidEuropeProjectConnector.Models;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepSendInfo : ActionStepBase
    {
        public const string Id = "FIELD_SENDINFO";
        public override string StepId { set; get; } = Id;
        public override AutoDictionary<string, string> StepLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Envio da informao" } };
        public override bool StepIsInput { get; set; } = false;

        private ICovidEuropeProjectInterface ExternalConnector { set; get; }

        public StepSendInfo(AnnaSession session) : base(session)
        {
            ExternalConnector = session.EmployeeFacade.ExternalConnectorsFactory<ICovidEuropeProjectInterface>(session, CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID);
        }

        public async override Task<List<StepResult>> ProcessBacktoken(Backtoken backtoken, WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();

            if (!await IsFulfilled())
            {
                switch (backtoken.Id)
                {
                    case Id + CovidEuropeProjectConsts.BKT_YES:
                        CovidData covidData = new CovidData()
                        {
                            geo_contConfirmed = StepValue(StepInfectedContact.Id).Value,
                            sympt_fever = StepValue(StepFever.Id).Value,
                            sympt_cough = StepValue(StepCoughing.Id).Value,
                            sympt_breath = StepValue(StepBreathingTrouble.Id).Value,
                        };

                        SaveStepData((await ExternalConnector.SendCovidResults(covidData)).ToString());
                        workerStatus.StepStatus = StepStatus.eMoveForward;
                        break;
                    case Id:
                        workerStatus.StepStatus = StepStatus.eMoveForward;
                        SaveStepData();
                        break;
                }
            }

            return results;
        }

        public override Task<List<StepResult>> GetStepQueryMessage(WorkerStatus workerStatus)
        {
            List<StepResult> listResults = new List<StepResult>();

            StepResult result = new StepResult
            {
                Message = "Por favor indique se os dados introduzidos correspondem a um caso real?"
            };

            result.HorizontalButtons.Add(
                StepResultUtils.CreateHButton(
                    Session.IntentControl,
                    CommonLocResources.Resources.YES,
                    Id + CovidEuropeProjectConsts.BKT_YES,
                    new List<string> { CovidEuropeProjectConsts.BKT_YES }));

            result.HorizontalButtons.Add(
                StepResultUtils.CreateHButton(
                    Session.IntentControl,
                    CommonLocResources.Resources.NO,
                    Id,
                    new List<string> { CovidEuropeProjectConsts.BKT_NO }));

            listResults.Add(result);

            return Task.FromResult(listResults);
        }
    }
}
