using Automaise.Core.Localizations;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Core.Entities.Flows;
using Automaise.Flows.IntentWorkers.ActionSteps;
using Automaise.Flows.Utils;
using System.Collections.Generic;
using System.Threading.Tasks;
using CovidEuropeProjectBase;
using Automaise.Core.DbEntities;
using System.Linq;
using Automaise.Core.Entities;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepTravelV2 : ActionStepEntityBase
    {
        public const string Id = "FIELD_TRAVEL";
        public override string StepId { set; get; } = Id;
        public const string Name = "Pas";
        public override AutoDictionary<string, string> StepLabel { get; set; } = new AutoDictionary<string, string> { { "pt", Name } };

        private ICovidEuropeProjectInterface ExternalConnector { set; get; }
        public override string EntityVariableName { get; set; } = "n/a";
        public override AutoDictionary<string, string> OptionButtonLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Indique Pas" } };
        public override bool Multichoice { get; set; } = true;
        public override bool ShowInlineOptions { get; set; } = false;

        public StepTravelV2(AnnaSession session) : base(session)
        {
            ExternalConnector = session.EmployeeFacade.ExternalConnectorsFactory<ICovidEuropeProjectInterface>(session, CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID);
            DisplayMessage = "Nos ltimos 14 dias, viajou para algum pas? Se sim qual?";
            ErrorMessage = CommonLocResources.Resources.INVALIDCOUNTRY;
        }

        public override async Task<List<NLPEntityValue>> OptionsListForStep()
        {
            List<ListItem> countries = await ExternalConnector.GetCountries();

            List<NLPEntityValue> entityValueList = new List<NLPEntityValue>();

            foreach (ListItem country in countries)
            {
                NLPEntityValue entityValue = new NLPEntityValue()
                {
                    Id = country.Id,
                    Name = new Dictionary<string, string>()
                };
                entityValue.Name.Add(AnnaCoreUtils.GetThreadLanguage(), country.Name);
                entityValueList.Add(entityValue);
            }

            NLPPartnerEntity entity = new NLPPartnerEntity
            {
                Values = entityValueList
            };

            if (entity == null || entity.Values.Count() == 0)
            {
                //We have to guarantee that there are infact entities to list!
                throw new System.ArgumentException("Empty countries list provided!");
            }

            return entity.Values;
        }

        public override async Task<List<StepResult>> GetStepQueryMessage(WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();

            StepResult result = new StepResult();

            FormState state = new FormState(GetType())
            {
                IntentControl = Session.IntentControl,
                MerchantId = Session.MerchantId,
                ChatId = Session.SessionId,
                Payload = Id
            };

            string optionsFormUrl = await WebPartsHelper.BuildWebPartURL(Session, WebPartsHelper.WebParts.eOptionPicker, state, false);

            result.Message = DisplayMessage;

            if (DefaultValues != null)
            {
                foreach (var item in DefaultValues[AnnaCoreUtils.GetThreadLanguage()])
                {
                    if (!string.IsNullOrWhiteSpace(item.Name))
                    {
                        result.VerticalButtons.Add(
                        StepResultUtils.CreateVButton(
                            Session.IntentControl,
                            item.Name,
                            StepId,
                            new List<string> { item.Id, item.Name }));
                    }
                }
            }

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButtonWithURL(
                    "Pases",
                    optionsFormUrl,
                    true,
                    StepResultUtils.WebViewSize.eCompact));

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButton(
                    Session.IntentControl,
                    CommonLocResources.Resources.NO,
                    Id,
                    new List<string> { CovidEuropeProjectConsts.BKT_NO },
                    MainIntents.AutomationFlow,
                    new List<FlowIntents> { FlowIntents.No, FlowIntents.GoBack }));

            results.Add(result);

            workerStatus.StepStatus = StepStatus.eMustReturn;

            return results;
        }
    }
}
