using Automaise.Core.Interfaces;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows.IntentWorkers;
using CovidEuropeProjectBase;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class ScreeningWorker : IntentWorkerBase
    {
        /// <summary>
        /// Flow started metric token
        /// </summary>
        public override string FlowStartedMetricToken { set; get; } = "Triagem Iniciada";

        /// <summary>
        /// Flow aborted metric token
        /// </summary>
        public override string FlowAbortedMetricToken { set; get; } = "Triagem Abortada";

        /// <summary>
        /// Flow completed metric token
        /// </summary>
        public override string FlowCompletedMetricToken { set; get; } = "Triagem Concluda";

        public ScreeningWorker() : base() { }

        public override Task<bool> AllowInlineIntentExecutions() => Task.FromResult(false);

        public override List<IActionStep> InitializeSteps(AnnaSession session)
        {
            List<IActionStep> actionSteps = new List<IActionStep>
            {
                new StepAge(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true },
            };
            if (CheckStepValue(session, StepAge.Id)
                && session.IntentStatus.ActionData.StepsGatheredData[StepAge.Id].Value.ToInt() < 18)
            {
                actionSteps.Add(new StepUserIsMinor(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepGender(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            actionSteps.Add(new StepPreExistantConditions(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            actionSteps.Add(new StepInfectedContact(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepInfectedContact.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepContactDays(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepFever(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepFever.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepFeverForMoreThan2Days(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepCoughing(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepCoughing.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepCoughingInterference(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
                actionSteps.Add(new StepCoughingBlood(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepBreathingTrouble(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepBreathingTrouble.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepBreathingTroubleScale(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepVomit(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepVomit.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepPersistentVomit(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepDiarrhea(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (CheckStepValueEquals(session, StepDiarrhea.Id, CovidEuropeProjectConsts.BKT_TRUE))
            {
                actionSteps.Add(new StepSevereDiarrhea(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepSymptoms(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            if (!NoSympthoms(session))
            {
                actionSteps.Add(new StepGlobalInterference(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            if (!HasSeverityLevels(session))
            {
                if (NoPreConditions(session))
                {
                    actionSteps.Add(new StepHasHomeIsolationConditions(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
                }
            }
            else if (!ClinicalForwarding(session))
            {
                actionSteps.Add(new StepHasHomeIsolationConditions(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });
            }
            actionSteps.Add(new StepScreeningSummary(session) { StepIsOptional = false, StepIsSilent = false, StepIsStrict = true });

            return actionSteps;
        }

        private bool NoSympthoms(AnnaSession session)
        {
            return (session.IntentStatus.ActionData.StepsGatheredData[StepSymptoms.Id] != null
                        && !session.IntentStatus.ActionData.StepsGatheredData[StepSymptoms.Id].Value.IsNullOrEmpty()
                        && (session.IntentStatus.ActionData.StepsGatheredData[StepSymptoms.Id].Value.Equals("Nenhum")
                            || (session.IntentStatus.ActionData.StepsGatheredData[StepSymptoms.Id].Value.Contains(":")
                                && session.IntentStatus.ActionData.StepsGatheredData[StepSymptoms.Id].Value.Split(":")[1].Split(";")[0].Equals("Nenhum")))
                    && (session.IntentStatus.ActionData.StepsGatheredData[StepCoughing.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepCoughing.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepCoughing.Id].Value.Equals(CovidEuropeProjectConsts.BKT_FALSE))
                    && (session.IntentStatus.ActionData.StepsGatheredData[StepFever.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepFever.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepFever.Id].Value.Equals(CovidEuropeProjectConsts.BKT_FALSE))
                    && (session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTrouble.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTrouble.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTrouble.Id].Value.Equals(CovidEuropeProjectConsts.BKT_FALSE))
                    && (session.IntentStatus.ActionData.StepsGatheredData[StepDiarrhea.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepDiarrhea.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepDiarrhea.Id].Value.Equals(CovidEuropeProjectConsts.BKT_FALSE))
                    && (session.IntentStatus.ActionData.StepsGatheredData[StepVomit.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepVomit.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepVomit.Id].Value.Equals(CovidEuropeProjectConsts.BKT_FALSE))
                    );
        }

        public bool ClinicalForwarding(AnnaSession session)
        {
            return (!NoPreConditions(session)
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id].Value.ToInt() == 4));
        }

        private bool HasSeverityLevels(AnnaSession session)
        {
            return ((session.IntentStatus.ActionData.StepsGatheredData[StepCoughingInterference.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepCoughingInterference.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepCoughingInterference.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepCoughingBlood.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepFeverForMoreThan2Days.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepSevereDiarrhea.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepPersistentVomit.Id].Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id] != null
                         && !session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id].Value.IsNullOrEmpty()
                         && session.IntentStatus.ActionData.StepsGatheredData[StepBreathingTroubleScale.Id].Value.ToInt() >= 2)
                    );
        }

        private bool CheckStepValueEquals(AnnaSession session, string stepId, string stepValue)
        {
            return session.IntentStatus.ActionData.StepsGatheredData[stepId] != null
                    && !session.IntentStatus.ActionData.StepsGatheredData[stepId].Value.IsNullOrEmpty()
                    && session.IntentStatus.ActionData.StepsGatheredData[stepId].Value.Equals(stepValue);
        }

        private bool CheckStepValue(AnnaSession session, string stepId)
        {
            return session.IntentStatus.ActionData.StepsGatheredData[stepId] != null
                    && !session.IntentStatus.ActionData.StepsGatheredData[stepId].Value.IsNullOrEmpty();
        }

        private bool NoPreConditions(AnnaSession session)
        {
            return (session.IntentStatus.ActionData.StepsGatheredData[StepPreExistantConditions.Id] != null
                     && !session.IntentStatus.ActionData.StepsGatheredData[StepPreExistantConditions.Id].Value.IsNullOrEmpty()
                     && (session.IntentStatus.ActionData.StepsGatheredData[StepPreExistantConditions.Id].Value.Contains("Nenhum")
                        || (session.IntentStatus.ActionData.StepsGatheredData[StepPreExistantConditions.Id].Value.Contains(":")
                           && session.IntentStatus.ActionData.StepsGatheredData[StepPreExistantConditions.Id].Value.Split(":")[1].Split(";")[0].Contains("Nenhum"))));
        }
    }
}
