using Automaise.Core.AI;
using Automaise.Core.Entities;
using Automaise.Core.Entities.Flows;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows.Base.ActionSteps;
using Automaise.Flows.IntentWorkers;
using CovidEuropeProjectConnector.Localizations;
using System.Collections.Generic;
using System.Threading.Tasks;
using static Automaise.Core.Session.AnnaSession;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepAge : StepNumberBase
    {
        public static string Id = "FIELD_AGE";
        public override string StepId { set; get; } = Id;
        public override AutoDictionary<string, string> StepLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Idade" },
                                                                                                                      { "es", "Edad" },
                                                                                                                      { "en", "Age" },
                                                                                                                      { "fr", "ge" },
                                                                                                                      { "de", "Alter" } };

        public StepAge(AnnaSession session) : base(session)
        {
            DisplayMessage = CovidEuropeProjectLocResources.Resources.ENTERAGE;
            ErrorMessage = CovidEuropeProjectLocResources.Resources.INVALIDAGE;
            MinValue = 0;
            MaxValue = 120;
        }

        public override Task<List<StepResult>> GetStepQueryMessage(WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();
            StepResult result;

            if (Session.IntentStatus.WorkerState == WorkerState.eUndefined)
            {
                result = new StepResult
                {
                    Message = IntentWorkerUtils.AddSentencePrefixIfNeeded(Session, CovidEuropeProjectLocResources.Resources.BEGINSCREENING),
                    WaitMsAfter = 1000
                };
                results.Add(result);
            }

            result = new StepResult
            {
                Message = DisplayMessage
            };

            results.Add(result);

            workerStatus.StepStatus = StepStatus.eMustReturn;

            return Task.FromResult(results);
        }

        public override async Task<List<StepResult>> ProcessMessage(string utterance, WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();
            bool isValid = false;

            if (!await IsFulfilled())
            {
                utterance = AnnaCoreUtils.NormalizeNumbersOnUtterance(utterance).Trim().ToLower();

                //Hack by EP to remove '19' from covid so the NLP doesn't confuse it with patient's age!
                utterance = utterance.Replace("covid-19", AnnaCoreUtils.GetRandomToken("covid-19".Length, true));
                utterance = utterance.Replace("covid_19", AnnaCoreUtils.GetRandomToken("covid_19".Length, true));
                utterance = utterance.Replace("covid19", AnnaCoreUtils.GetRandomToken("covid19".Length, true));
                utterance = utterance.Replace("covid 19", AnnaCoreUtils.GetRandomToken("covid 19".Length, true));

                var parserResult = await PowerParserService.NLPUtteranceForEntityType(utterance, EntityType.Number);

                if (parserResult.Count == 0)
                {
                    parserResult = await PowerParserService.NLPUtteranceForEntityType(utterance, EntityType.Duration);
                }

                if (parserResult.Count > 0)
                {
                    foreach (var duckres in parserResult)
                    {
                        var normalizeValue = AnnaCoreUtils.NormalizeNumbersOnUtterance(duckres.body).Trim();

                        int valueToBeCompared = DigitsCountMode == true ? normalizeValue.Length : duckres.value.value.ToInt();

                        if (MinValue == null && MaxValue != null)
                        {
                            if (valueToBeCompared <= MaxValue)
                            {
                                isValid = true;
                            }
                        }
                        else if (MinValue != null && MaxValue == null)
                        {
                            if (valueToBeCompared >= MinValue)
                            {
                                isValid = true;
                            }
                        }
                        else if (MinValue != null && MaxValue != null)
                        {
                            if (valueToBeCompared >= MinValue && valueToBeCompared <= MaxValue)
                            {
                                isValid = true;
                            }
                        }
                        else
                        {
                            isValid = true;
                        }

                        if (isValid)
                        {
                            SaveStepData(duckres.value.value, duckres.start, duckres.end);
                            workerStatus.StepStatus = StepStatus.eMoveForward;
                            break;
                        }
                        else
                        {
                            workerStatus.StepStatus = StepStatus.eError;
                        }
                    }
                }
                else if (StepValue(StepId).Value.IsNullOrWhiteSpace())
                {
                    workerStatus.StepStatus = StepStatus.eError;
                }
            }

            return results;
        }
    }
}