using Automaise.Core;
using Automaise.Core.Entities;
using Automaise.Core.Entities.Flows;
using Automaise.Core.Interfaces;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows.IntentWorkers.ActionSteps;
using Automaise.Flows.Utils;
using CovidEuropeProjectBase;
using CovidEuropeProjectConnector.Localizations;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepBreathingTroubleScale : ActionStepListBase
    {
        public static string Id { get; set; } = "FIELD_TROUBLE_BREATHING_SCALE";
        public override string StepId { set; get; } = Id;
        public override AutoDictionary<string, string> StepLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Escala de falta de ar" },
                                                                                                                      { "es", "Escala de dificultad para respirar" },
                                                                                                                      { "en", "Shortness of breath scale" },
                                                                                                                      { "fr", "chelle d'essoufflement" },
                                                                                                                      { "de", "Kurzatmigkeit der atemskala" } };
        public override bool StepIsInput { get; set; } = false;
        private ICovidEuropeProjectInterface ExternalConnector { set; get; }
        public override bool Multichoice { get; set; } = false;
        public override bool ShowInlineOptions { get; set; } = false;

        public StepBreathingTroubleScale(AnnaSession session) : base(session) 
        { 
            ExternalConnector = session.EmployeeFacade.ExternalConnectorsFactory<ICovidEuropeProjectInterface>(session, CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID);
        }

        public override async Task<List<StepResult>> GetStepQueryMessage(WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();

            StepResult result = new StepResult
            {
                Message = CovidEuropeProjectLocResources.Resources.TROUBLEBREATHINGSCALE
            };

            FormState state = new FormState(GetType())
            {
                IntentControl = Session.IntentControl,
                MerchantId = Session.MerchantId,
                ChatId = Session.SessionId,
                MsgType = MessageType.eUndefined,
                Payload = Id,
                Parameters = new Dictionary<string, string>
                    {
                        { "optoken", "SCALE" },
                    }
            };

            string environment = CovidEuropeProjectEnv.Settings.CUSTOM_PARTS_ENVIRONMENTS[AutomaiseEnv.Settings.RunningEnv];

            string optionsFormUrl = await WebPartsHelper.BuildCustomWebPartURL(Session, environment + "optionsform", "SCALE", state, true);

            result.VerticalButtons.Add(
                StepResultUtils.CreateVButtonWithURL(
                        CovidEuropeProjectLocResources.Resources.SCALE,
                        optionsFormUrl));

            results.Add(result);

            return results;
        }

        public override async Task<List<IListItem>> OptionsListForStep(AnnaSession session)
        {
            var scale = await ExternalConnector.MRCQuestionaire();
            return scale.Select(p => new ListItem { Id = p.Id, Name = p.Name }).ToList<IListItem>();
        }
    }
}
