using Automaise.Core.AI;
using Automaise.Core.DbEntities;
using Automaise.Core.Session;
using Automaise.Core.Utils;
using Automaise.Flows.IntentWorkers.ActionSteps;
using CovidEuropeProjectConnector.Localizations;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepPreExistantConditions : ActionStepEntityBase
    {
        public const string Id = "FIELD_PRE_EXISTANT_CONDITIONS";
        public override string StepId { set; get; } = Id;
        public override AutoDictionary<string, string> StepLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Condies prvias" },
                                                                                                                      { "en", "Preconditions" },
                                                                                                                      { "es", "Condiciones previas" },
                                                                                                                      { "fr", "Conditions pralables" },
                                                                                                                      { "de", "Voraussetzungen" } };

        public override string EntityVariableName { get; set; } = "Conditions";
        public override AutoDictionary<string, string> OptionButtonLabel { get; set; } = new AutoDictionary<string, string> { { "pt", "Condies prvias" },
                                                                                                                              { "en", "Preconditions" },
                                                                                                                              { "es", "Condiciones previas" },
                                                                                                                              { "fr", "Conditions pralables" },
                                                                                                                              { "de", "Voraussetzungen" }};
        public override bool Multichoice { get; set; } = true;
        public override bool ShowInlineOptions { get ; set; } = false;
        public override bool FilterByMerchantId { get; set; } = false;

        public StepPreExistantConditions(AnnaSession session) : base(session)
        {
            DisplayMessage = CovidEuropeProjectLocResources.Resources.PREEXISTENTCONDITIONS;
            ErrorMessage = CovidEuropeProjectLocResources.Resources.INVALIDCONDITION;
        }

        public override async Task<List<NLPEntityValue>> OptionsListForStep()
        {
            NLPPartnerEntity entity = await NLPEntityService.GetEntityData(Session.MerchantObj, EntityVariableName, FilterByMerchantId);

            if (entity == null || entity.Values.Count() == 0)
            {
                //We have to guarantee that there are infact entities to list!
                throw new System.ArgumentException($"The given EntityVariableName \"{EntityVariableName}\" it's not valid or doesn't have any content!");
            }

            List<NLPEntityValue> values = entity.Values;

            if (StepValue(StepGender.Id) != null
                && !StepValue(StepGender.Id).Value.IsNullOrWhiteSpace()
                && !StepValue(StepGender.Id).Value.Equals(CovidEuropeProjectLocResources.Resources.FEMININE))
            {
                values = values.AsQueryable().Where(x => !x.Name.Equals(CovidEuropeProjectLocResources.Resources.PREGNANCY)).ToList();
            }

            return values;
        }
    }
}
