using Automaise.Core.Session;
using Automaise.Flows.IntentWorkers.ActionSteps;
using Automaise.Core.Utils;
using Automaise.Core.Entities.Flows;
using System.Collections.Generic;
using System.Threading.Tasks;
using CovidEuropeProjectBase;
using Automaise.Flows;
using Automaise.Core.Outbound;
using CovidEuropeProjectConnector.Models;
using Automaise.Core.Services;
using Automaise.Core.ClientDB;
using CovidEuropeProjectConnector.Localizations;
using System;

namespace CovidEuropeProjectEdge.CovidEuropeProject.Workers
{
    public class StepScreeningSummary : ActionStepBase
    {

        public static string Id = "FIELD_SCREENINGSUMARY";
        public override string StepId { set; get; } = "Id";
        public override AutoDictionary<string, string> StepLabel { set; get; } = new AutoDictionary<string, string> { { "pt", "Diagnstico da triagem" },
                                                                                                                      { "en", "Screening Diagnosis" },
                                                                                                                      { "es", "Diagnstico de la deteccin" },
                                                                                                                      { "fr", "Diagnostic du dpistage" },
                                                                                                                      { "de", "Diagnose" } };
        public override bool BaseAIProcessing { get; set; } = false;
        public override bool StepIsInput { get; set; } = false;
        private ICovidEuropeProjectInterface ExternalConnector { set; get; }

        public StepScreeningSummary(AnnaSession session) : base(session)
        {
            ExternalConnector = session.EmployeeFacade.ExternalConnectorsFactory<ICovidEuropeProjectInterface>(session, CovidEuropeProjectConsts.VIRTUAL_EMPLOYEE_ID);
        }

        public async override Task<List<StepResult>> GetStepQueryMessage(WorkerStatus workerStatus)
        {
            List<StepResult> results = new List<StepResult>();

            MildSymptoms mildSymptoms = GetSymptoms();

            CovidDataV2 covidData = new CovidDataV2()
            {
                bio_hemodinamic = false,
                bio_systolic_blood_pressure = 0,
                bio_diastolic_blood_pressure = 0,
                bio_heart_rate = 0,
                bio_o2saturation = 0,
                bio_age = ReturnInteger(StepAge.Id),
                bio_sex = ConvertGender(StepValue(StepGender.Id).Value),
                bio_comorbidities = !NoPreConditions(),
                geo_condition_home_isolation = ReturnTrueOrFalse(StepHasHomeIsolationConditions.Id, true),
                sympt_respiratory = new SymptRespiratory
                {
                    scale_MRC = ReturnInteger(StepBreathingTroubleScale.Id),
                    frequency = ReturnInteger(StepBreathingTroubleScale.Id) * 10,
                    breathlessness = ReturnTrueOrFalse(StepBreathingTrouble.Id)
                },
                sympt_fever = new SymptFever
                {
                    has_fever = ReturnTrueOrFalse(StepFever.Id),
                    persistent = ReturnTrueOrFalse(StepFeverForMoreThan2Days.Id),
                    reappearing = ReturnTrueOrFalse(StepFeverForMoreThan2Days.Id)
                },
                sympt_cough = new SymptCough
                {
                    blood = ReturnTrueOrFalse(StepCoughingBlood.Id),
                    persistent = ReturnTrueOrFalse(StepCoughing.Id),
                    interfering = ReturnTrueOrFalse(StepCoughingInterference.Id)
                },
                clinical_forwarding = (ClinicalForwarding()
                                        || mildSymptoms.ChestPain
                                        || mildSymptoms.AlteredConscience),
                geo_contact = new GeoContact
                {
                    confirmed = ReturnTrueOrFalse(StepInfectedContact.Id),
                    hard_suspect = ReturnTrueOrFalse(StepInfectedContact.Id),
                    days = ReturnInteger(StepContactDays.Id)
                },
                sympt_global_interference = ReturnInteger(StepGlobalInterference.Id),
                sympt_vomit = new SymptVomit
                {
                    has_vomit = ReturnTrueOrFalse(StepVomit.Id),
                    persistent = ReturnTrueOrFalse(StepPersistentVomit.Id)
                },
                sympt_diarrhea = new SymptDiarrhea
                {
                    has_diarrhea = ReturnTrueOrFalse(StepDiarrhea.Id),
                    severe = ReturnTrueOrFalse(StepSevereDiarrhea.Id)
                },
                sympt_altered_consciouness = mildSymptoms.AlteredConscience,
                sympt_chest_pain = mildSymptoms.ChestPain,
                sympt_headache = mildSymptoms.Headache,
                sympt_loss_olfact = mildSymptoms.LossOlfact,
                sympt_loss_palate = mildSymptoms.LossPalate,
                sympt_muscle_pain = mildSymptoms.MusclePain
            };

            CovidResult covidResult = await ExternalConnector.SendCovidResultsV2(covidData);

            PersistentStorageService persistentStorageService = new PersistentStorageService(Session.MerchantId);

            persistentStorageService.TryAddValue(
                CovidEuropeProjectConsts.COVID_RESULTS_DATA,
                $"{Session.SessionId}_{DateTime.Now}",
                new CovidResultsStruct { CovidData = covidData, CovidResult = covidResult });

            StepResult result;

            if (Session.InboundUser.ChatClientId == InboundChannel.FacebookMessenger)
            {
                result = new StepResult
                {
                    Message = "*" + ConvertLevelToWarningMessage(covidResult.level) + "*",
                    WaitMsAfter = 1000
                };

                results.Add(result);
            }

            result = new StepResult
            {
                Message = AnnaCoreUtils.RemoveLineBreaks(covidResult.message),
                ResultColor = ConvertLevelToColor(covidResult.level),
                WaitMsAfter = 3000
            };

            results.Add(result);

            SaveStepData();

            workerStatus.WorkerGlobalStatus = WorkerGlobalStatus.WorkerIsComplete_ASK_WhatElse;
            workerStatus.StepStatus = StepStatus.eDone;

            switch (covidResult.level)
            {
                case 1:
                    MetricsService.METRIC(Session.MerchantId, Session?.SessionId, CovidEuropeProjectConsts.RECOMMEND_PREVENTION, MetricType.Default, Session);
                    break;
                case 2:
                    MetricsService.METRIC(Session.MerchantId, Session?.SessionId, CovidEuropeProjectConsts.RECOMMEND_MONITORING, MetricType.Default, Session);
                    break;
                case 3:
                    MetricsService.METRIC(Session.MerchantId, Session?.SessionId, CovidEuropeProjectConsts.RECOMMEND_ACTION, MetricType.Default, Session);
                    break;
                default:
                    break;
            }

            return results;
        }

        private int ConvertGender(string gender)
        {
            var genderValue = gender switch
            {
                "Masculino" => 1,
                "Feminino" => 2,
                _ => 0,
            };

            return genderValue;
        }

        public class CovidResultsStruct
        {
            public CovidResult CovidResult { get; set; }
            public CovidDataV2 CovidData { get; set; }
        }

        private string ConvertLevelToWarningMessage(int level)
        {
            string message = null;

            switch (level)
            {
                case 1:
                    message = CovidEuropeProjectLocResources.Resources.PREVENTION;
                    break;
                case 2:
                    message = CovidEuropeProjectLocResources.Resources.MONITORING;
                    break;
                case 3:
                    message = CovidEuropeProjectLocResources.Resources.ACTION;
                    break;
                default:
                    break;
            }

            return message;
        }

        private MildSymptoms GetSymptoms()
        {
            MildSymptoms mildSymptoms = new MildSymptoms();

            string[] firstSymptom;

            string[] result = StepValue(StepSymptoms.Id).Value.Split(":");

            bool hasSympthoms = false;

            if (result.Length > 1)
            {
                firstSymptom = result[1].Split(";");
                if (!firstSymptom[0].Contains(CovidEuropeProjectLocResources.Resources.NONE))
                {
                    MapSymptoms(mildSymptoms, firstSymptom[0], firstSymptom[1].ToBool());
                    hasSympthoms = true;
                }
            }
            else
            {
                if (!StepValue(StepSymptoms.Id).Value.Contains(CovidEuropeProjectLocResources.Resources.NONE))
                {
                    MapSymptoms(mildSymptoms, StepValue(StepSymptoms.Id).Value, true);
                    hasSympthoms = true;
                }
            }

            if (hasSympthoms)
            {
                for (int i = 1; i < 20; i++)
                {
                    if (StepContainsKey(StepSymptoms.Id + "_" + StepSymptoms.Id + $"_{i}"))
                    {
                        string[] item = StepValue(StepSymptoms.Id + "_" + StepSymptoms.Id + $"_{i}").Value.Split(":")[1].Split(";");
                        MapSymptoms(mildSymptoms, item[0], item[1].ToBool());
                    }
                    else
                    {
                        break;
                    }
                }
            }

            return mildSymptoms;

        }

        private string ConvertLevelToColor(int level)
        {
            string color = null;

            switch (level)
            {
                case 1:
                    color = CovidEuropeProjectConsts.COLOR_PREVENTION;
                    break;
                case 2:
                    color = CovidEuropeProjectConsts.COLOR_MONITORING;
                    break;
                case 3:
                    color = CovidEuropeProjectConsts.COLOR_ACTION;
                    break;
                default:
                    break;
            }

            return color;
        }

        private void MapSymptoms(MildSymptoms mildSymptoms, string symptom, bool value)
        {
            switch (symptom)
            {
                case "Dores de cabea":
                    mildSymptoms.Headache = value;
                    break;
                case "Perda de olfato":
                    mildSymptoms.LossOlfact = value;
                    break;
                case "Perda de paladar":
                    mildSymptoms.LossPalate = value;
                    break;
                case "Dores musculares":
                    mildSymptoms.MusclePain = value;
                    break;
                case "Dor no peito quando respira":
                    mildSymptoms.ChestPain = value;
                    break;
                case "Dificuldade em organizar pensamento":
                    mildSymptoms.AlteredConscience = value;
                    break;
            }
        }

        private bool ReturnTrueOrFalse(string stepId, bool overrideBoolean = false)
        {
            return !StepValue(stepId).Value.IsNullOrEmpty()
                   ? StepValue(stepId).Value.ToBool()
                   : overrideBoolean;
        }

        private int ReturnInteger(string stepId)
        {
            return !StepValue(stepId).Value.IsNullOrEmpty()
                   ? StepValue(stepId).Value.ToInt()
                   : 0;
        }

        private bool ClinicalForwarding()
        {
            return (!NoPreConditions()
                    || (StepContainsKey(StepCoughingBlood.Id)
                         && !StepValue(StepCoughingBlood.Id).Value.IsNullOrEmpty()
                         && StepValue(StepCoughingBlood.Id).Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (StepContainsKey(StepFeverForMoreThan2Days.Id)
                         && !StepValue(StepFeverForMoreThan2Days.Id).Value.IsNullOrEmpty()
                         && StepValue(StepFeverForMoreThan2Days.Id).Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (StepContainsKey(StepPersistentVomit.Id)
                         && !StepValue(StepPersistentVomit.Id).Value.IsNullOrEmpty()
                         && StepValue(StepPersistentVomit.Id).Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (StepContainsKey(StepSevereDiarrhea.Id)
                         && !StepValue(StepSevereDiarrhea.Id).Value.IsNullOrEmpty()
                         && StepValue(StepSevereDiarrhea.Id).Value.Equals(CovidEuropeProjectConsts.BKT_TRUE))
                    || (StepContainsKey(StepBreathingTroubleScale.Id)
                         && !StepValue(StepBreathingTroubleScale.Id).Value.IsNullOrEmpty()
                         && StepValue(StepBreathingTroubleScale.Id).Value.ToInt() == 4));
        }

        private bool NoPreConditions()
        {
            return (StepContainsKey(StepPreExistantConditions.Id)
                     && !StepValue(StepPreExistantConditions.Id).Value.IsNullOrEmpty()
                     && (StepValue(StepPreExistantConditions.Id).Value.Contains("Nenhum")
                        || (StepValue(StepPreExistantConditions.Id).Value.Contains(":")
                            && StepValue(StepPreExistantConditions.Id).Value.Split(":")[1].Split(";")[0].Contains("Nenhum"))));
        }
    }
}
